@@ -23,6 +23,19 @@ SEXP debug_XStringSet_io()
 #define IOBUF_SIZE 20002
 static char errmsg_buf[200];
 
+static int has_prefix(const char *s, const char *prefix)
+{
+	int i = 0;
+	char c;
+
+	while ((c = prefix[i]) != '\0') {
+		if (s[i] != c)
+			return 0;
+		i++;
+	}
+	return 1;
+}
+
 
 
 /****************************************************************************
@@ -31,7 +44,7 @@ static char errmsg_buf[200];
 
 /*
  * Even if the standard FASTA markup is made of 1-letter sympols, we want to
- * be able to eventually support longer sympols.
+ * be ready to support longer sympols just in case.
  */
 static const char *FASTA_comment_markup = ";", *FASTA_desc_markup = ">";
 
@@ -44,10 +57,10 @@ typedef struct fasta_loader {
 	const int *lkup;
 	int lkup_length;
 	void (*load_desc_line)(struct fasta_loader *loader,
-			       const cachedCharSeq *desc_line);
+			       const Chars_holder *desc_line);
 	void (*load_empty_seq)(struct fasta_loader *loader);
 	void (*load_seq_data)(struct fasta_loader *loader,
-			      const cachedCharSeq *seq_data);
+			      const Chars_holder *seq_data);
 	int nrec;
 	void *ext;  /* loader extension (optional) */
 } FASTAloader;
@@ -63,7 +76,7 @@ typedef struct fastainfo_loader_ext {
 } FASTAINFO_loaderExt;
 
 static void FASTAINFO_load_desc_line(FASTAloader *loader,
-		const cachedCharSeq *desc_line)
+		const Chars_holder *desc_line)
 {
 	FASTAINFO_loaderExt *loader_ext;
 	CharAEAE *ans_names_buf;
@@ -87,7 +100,7 @@ static void FASTAINFO_load_empty_seq(FASTAloader *loader)
 }
 
 static void FASTAINFO_load_seq_data(FASTAloader *loader,
-		const cachedCharSeq *seq_data)
+		const Chars_holder *seq_data)
 {
 	FASTAINFO_loaderExt *loader_ext;
 	IntAE *seqlengths_buf;
@@ -115,6 +128,7 @@ static FASTAloader new_FASTAINFO_loader(SEXP lkup, int load_descs,
 
 	if (lkup == R_NilValue) {
 		loader.lkup = NULL;
+		loader.lkup_length = 0;
 	} else {
 		loader.lkup = INTEGER(lkup);
 		loader.lkup_length = LENGTH(lkup);
@@ -132,36 +146,36 @@ static FASTAloader new_FASTAINFO_loader(SEXP lkup, int load_descs,
  */
 
 typedef struct fasta_loader_ext {
-	cachedXVectorList cached_ans;
-	cachedCharSeq cached_ans_elt;
+	XVectorList_holder ans_holder;
+	Chars_holder ans_elt_holder;
 } FASTA_loaderExt;
 
 static void FASTA_load_empty_seq(FASTAloader *loader)
 {
 	FASTA_loaderExt *loader_ext;
-	cachedCharSeq *cached_ans_elt;
+	Chars_holder *ans_elt_holder;
 
 	loader_ext = loader->ext;
-	cached_ans_elt = &(loader_ext->cached_ans_elt);
-	*cached_ans_elt = get_cachedXRawList_elt(&(loader_ext->cached_ans),
+	ans_elt_holder = &(loader_ext->ans_elt_holder);
+	*ans_elt_holder = get_elt_from_XRawList_holder(&(loader_ext->ans_holder),
 						 loader->nrec);
-	cached_ans_elt->length = 0;
+	ans_elt_holder->length = 0;
 	return;
 }
 
 static void FASTA_load_seq_data(FASTAloader *loader,
-		const cachedCharSeq *seq_data)
+		const Chars_holder *seq_data)
 {
 	FASTA_loaderExt *loader_ext;
-	cachedCharSeq *cached_ans_elt;
+	Chars_holder *ans_elt_holder;
 
 	loader_ext = loader->ext;
-	cached_ans_elt = &(loader_ext->cached_ans_elt);
-	/* cached_ans_elt->seq is a const char * so we need to cast it to
+	ans_elt_holder = &(loader_ext->ans_elt_holder);
+	/* ans_elt_holder->seq is a const char * so we need to cast it to
 	   char * in order to write to it */
-	memcpy((char *) cached_ans_elt->seq + cached_ans_elt->length,
+	memcpy((char *) ans_elt_holder->seq + ans_elt_holder->length,
 	       seq_data->seq, seq_data->length * sizeof(char));
-	cached_ans_elt->length += seq_data->length;
+	ans_elt_holder->length += seq_data->length;
 	return;
 }
 
@@ -169,7 +183,7 @@ static FASTA_loaderExt new_FASTA_loaderExt(SEXP ans)
 {
 	FASTA_loaderExt loader_ext;
 
-	loader_ext.cached_ans = cache_XVectorList(ans);
+	loader_ext.ans_holder = hold_XVectorList(ans);
 	return loader_ext;
 }
 
@@ -179,6 +193,7 @@ static FASTAloader new_FASTA_loader(SEXP lkup, FASTA_loaderExt *loader_ext)
 
 	if (lkup == R_NilValue) {
 		loader.lkup = NULL;
+		loader.lkup_length = 0;
 	} else {
 		loader.lkup = INTEGER(lkup);
 		loader.lkup_length = LENGTH(lkup);
@@ -191,7 +206,7 @@ static FASTAloader new_FASTA_loader(SEXP lkup, FASTA_loaderExt *loader_ext)
 	return loader;
 }
 
-static int translate(cachedCharSeq *seq_data, const int *lkup, int lkup_length)
+static int translate(Chars_holder *seq_data, const int *lkup, int lkup_length)
 {
 	char *dest;
 	int nbinvalid, i, j, key, val;
@@ -216,34 +231,47 @@ static int translate(cachedCharSeq *seq_data, const int *lkup, int lkup_length)
  * Ignore empty lines and lines starting with 'FASTA_comment_markup' like in
  * the original Pearson FASTA format.
  */
-static const char *parse_FASTA_file(FILE *stream, int *recno, int *ninvalid,
-		int nrec, int skip, FASTAloader *loader)
+static const char *parse_FASTA_file(SEXP efp, int *recno, int *ninvalid,
+		int nrec, int skip, int seek_first_rec, FASTAloader *loader)
 {
-	int FASTA_comment_markup_length, FASTA_desc_markup_length,
-	    lineno, previous_line_not_complete, load_record,
-	    current_line_not_complete, new_record;
+	int lineno, EOL_in_buf, EOL_in_prev_buf, ret_code,
+	    FASTA_desc_markup_length, load_rec, is_new_rec;
 	char buf[IOBUF_SIZE];
-	cachedCharSeq data;
+	Chars_holder data;
 
-	FASTA_comment_markup_length = strlen(FASTA_comment_markup);
 	FASTA_desc_markup_length = strlen(FASTA_desc_markup);
-	lineno = 0;
-	previous_line_not_complete = 0;
-	load_record = -1;
-	while (fgets(buf, IOBUF_SIZE, stream) != NULL) {
-		data.length = delete_trailing_LF_or_CRLF(buf, -1);
+	load_rec = -1;
+	for (lineno = EOL_in_prev_buf = 1;
+	     (ret_code = ExternalFilePtr_gets(efp, buf, IOBUF_SIZE,
+					      &EOL_in_buf));
+	     lineno += EOL_in_prev_buf = EOL_in_buf)
+	{
+		if (ret_code == -1) {
+			snprintf(errmsg_buf, sizeof(errmsg_buf),
+				 "read error while reading characters "
+				 "from line %d", lineno);
+			return errmsg_buf;
+		}
+		if (seek_first_rec) {
+			if (EOL_in_prev_buf
+			 && has_prefix(buf, FASTA_desc_markup)) {
+				seek_first_rec = 0;
+			} else {
+				continue;
+			}
+		}
+		if (EOL_in_buf) {
+			data.length = delete_trailing_LF_or_CRLF(buf, -1);
+		} else {
+			data.length = IOBUF_SIZE - 1;
+		}
 		data.seq = buf;
-		// data.length >= IOBUF_SIZE should never happen
-		current_line_not_complete = data.length >= IOBUF_SIZE - 1;
-		if (previous_line_not_complete)
+		if (!EOL_in_prev_buf)
 			goto parse_seq_data;
-		lineno++;
 		if (data.length == 0)
 			continue; // we ignore empty lines
-		if (data.length >= FASTA_comment_markup_length &&
-		    strncmp(data.seq, FASTA_comment_markup,
-			    FASTA_comment_markup_length) == 0) {
-			if (current_line_not_complete) {
+		if (has_prefix(data.seq, FASTA_comment_markup)) {
+			if (!EOL_in_buf) {
 				snprintf(errmsg_buf, sizeof(errmsg_buf),
 					 "cannot read line %d, "
 					 "line is too long", lineno);
@@ -252,71 +280,75 @@ static const char *parse_FASTA_file(FILE *stream, int *recno, int *ninvalid,
 			continue; // we ignore comment lines
 		}
 		buf[data.length] = '\0';
-		new_record = strncmp(data.seq, FASTA_desc_markup,
-				     FASTA_desc_markup_length) == 0;
-		if (new_record) {
-			if (current_line_not_complete) {
+		is_new_rec = has_prefix(data.seq, FASTA_desc_markup);
+		if (is_new_rec) {
+			if (!EOL_in_buf) {
 				snprintf(errmsg_buf, sizeof(errmsg_buf),
 					 "cannot read line %d, "
 					 "line is too long", lineno);
 				return errmsg_buf;
 			}
-			load_record = *recno >= skip;
-			if (load_record && nrec >= 0 && *recno >= skip + nrec)
+			load_rec = *recno >= skip;
+			if (load_rec && nrec >= 0 && *recno >= skip + nrec)
 				return NULL;
-			load_record = load_record && loader != NULL;
-			if (load_record && loader->load_desc_line != NULL) {
+			load_rec = load_rec && loader != NULL;
+			if (load_rec && loader->load_desc_line != NULL) {
 				data.seq += FASTA_desc_markup_length;
 				data.length -= FASTA_desc_markup_length;
 				loader->load_desc_line(loader, &data);
 			}
-			if (load_record && loader->load_empty_seq != NULL)
+			if (load_rec && loader->load_empty_seq != NULL)
 				loader->load_empty_seq(loader);
-			if (load_record)
+			if (load_rec)
 				loader->nrec++;
 			(*recno)++;
 			continue;
 		}
-		if (load_record == -1) {
+		if (load_rec == -1) {
 			snprintf(errmsg_buf, sizeof(errmsg_buf),
 				 "\"%s\" expected at beginning of line %d",
 				 FASTA_desc_markup, lineno);
 			return errmsg_buf;
 		}
 		parse_seq_data:
-		if (load_record && loader->load_seq_data != NULL) {
+		if (load_rec && loader->load_seq_data != NULL) {
 			if (loader->lkup != NULL)
 				*ninvalid += translate(&data,
 						       loader->lkup,
 						       loader->lkup_length);
 			loader->load_seq_data(loader, &data);
 		}
-		previous_line_not_complete = current_line_not_complete;
+	}
+	if (seek_first_rec) {
+		snprintf(errmsg_buf, sizeof(errmsg_buf),
+			 "no FASTA record found");
+		return errmsg_buf;
 	}
 	return NULL;
 }
 
 /* --- .Call ENTRY POINT --- */
-SEXP fasta_info(SEXP efp_list, SEXP nrec, SEXP skip, SEXP use_names, SEXP lkup)
+SEXP fasta_info(SEXP efp_list, SEXP nrec, SEXP skip, SEXP seek_first_rec,
+		SEXP use_names, SEXP lkup)
 {
-	int nrec0, skip0, load_descs, i, recno, ninvalid;
+	int nrec0, skip0, seek_rec0, load_descs, i, recno, ninvalid;
 	FASTAINFO_loaderExt loader_ext;
 	FASTAloader loader;
-	FILE *stream;
-	SEXP ans, ans_names;
+	SEXP efp, ans, ans_names;
 	const char *errmsg;
 
 	nrec0 = INTEGER(nrec)[0];
 	skip0 = INTEGER(skip)[0];
+	seek_rec0 = LOGICAL(seek_first_rec)[0];
 	load_descs = LOGICAL(use_names)[0];
 	loader_ext = new_FASTAINFO_loaderExt();
 	loader = new_FASTAINFO_loader(lkup, load_descs, &loader_ext);
 	recno = 0;
 	for (i = 0; i < LENGTH(efp_list); i++) {
-		stream = R_ExternalPtrAddr(VECTOR_ELT(efp_list, i));
+		efp = VECTOR_ELT(efp_list, i);
 		ninvalid = 0;
-		errmsg = parse_FASTA_file(stream, &recno, &ninvalid,
-					  nrec0, skip0, &loader);
+		errmsg = parse_FASTA_file(efp, &recno, &ninvalid,
+					  nrec0, skip0, seek_rec0, &loader);
 		if (errmsg != NULL)
 			error("reading FASTA file %s: %s",
 			      CHAR(STRING_ELT(GET_NAMES(efp_list), i)),
@@ -340,19 +372,21 @@ SEXP fasta_info(SEXP efp_list, SEXP nrec, SEXP skip, SEXP use_names, SEXP lkup)
 
 /* --- .Call ENTRY POINT --- */
 SEXP read_fasta_in_XStringSet(SEXP efp_list, SEXP nrec, SEXP skip,
+		SEXP seek_first_rec,
 		SEXP use_names, SEXP elementType, SEXP lkup)
 {
-	int nrec0, skip0, i, recno, ninvalid;
-	SEXP ans_width, ans_names, ans;
+	int nrec0, skip0, seek_rec0, i, recno, ninvalid;
+	SEXP efp, ans_width, ans_names, ans;
 	const char *element_type;
 	char classname[40];  /* longest string should be "DNAStringSet" */
 	FASTA_loaderExt loader_ext;
 	FASTAloader loader;
-	FILE *stream;
 
 	nrec0 = INTEGER(nrec)[0];
 	skip0 = INTEGER(skip)[0];
-	PROTECT(ans_width = fasta_info(efp_list, nrec, skip, use_names, lkup));
+	seek_rec0 = LOGICAL(seek_first_rec)[0];
+	PROTECT(ans_width = fasta_info(efp_list, nrec, skip, seek_first_rec,
+				       use_names, lkup));
 	PROTECT(ans_names = GET_NAMES(ans_width));
 	SET_NAMES(ans_width, R_NilValue);
 	element_type = CHAR(STRING_ELT(elementType, 0));
@@ -370,10 +404,10 @@ SEXP read_fasta_in_XStringSet(SEXP efp_list, SEXP nrec, SEXP skip,
 	loader = new_FASTA_loader(lkup, &loader_ext);
 	recno = ninvalid = 0;
 	for (i = 0; i < LENGTH(efp_list); i++) {
-		stream = R_ExternalPtrAddr(VECTOR_ELT(efp_list, i));
-		rewind(stream);
-		parse_FASTA_file(stream, &recno, &ninvalid,
-				 nrec0, skip0, &loader);
+		efp = VECTOR_ELT(efp_list, i);
+		ExternalFilePtr_rewind(efp);
+		parse_FASTA_file(efp, &recno, &ninvalid,
+				 nrec0, skip0, seek_rec0, &loader);
 	}
 	UNPROTECT(3);
 	return ans;
@@ -387,17 +421,16 @@ SEXP read_fasta_in_XStringSet(SEXP efp_list, SEXP nrec, SEXP skip,
 /* --- .Call ENTRY POINT --- */
 SEXP write_XStringSet_to_fasta(SEXP x, SEXP efp_list, SEXP width, SEXP lkup)
 {
-	cachedXStringSet X;
+	XStringSet_holder X;
 	int x_length, width0, lkup_length, i, j1, j2, dest_nbytes;
-	FILE *stream;
 	const int *lkup0;
-	SEXP x_names, desc;
-	cachedCharSeq X_elt;
+	SEXP efp, x_names, desc;
+	Chars_holder X_elt;
 	char buf[IOBUF_SIZE];
 
-	X = _cache_XStringSet(x);
-	x_length = _get_cachedXStringSet_length(&X);
-	stream = R_ExternalPtrAddr(VECTOR_ELT(efp_list, 0));
+	X = _hold_XStringSet(x);
+	x_length = _get_length_from_XStringSet_holder(&X);
+	efp = VECTOR_ELT(efp_list, 0);
 	width0 = INTEGER(width)[0];
 	if (width0 >= IOBUF_SIZE)
 		error("'width' must be <= %d", IOBUF_SIZE - 1);
@@ -411,18 +444,15 @@ SEXP write_XStringSet_to_fasta(SEXP x, SEXP efp_list, SEXP width, SEXP lkup)
 	}
 	x_names = get_XVectorList_names(x);
 	for (i = 0; i < x_length; i++) {
-		if (fputs(FASTA_desc_markup, stream) == EOF)
-			error("write error");
+		ExternalFilePtr_puts(efp, FASTA_desc_markup);
 		if (x_names != R_NilValue) {
 			desc = STRING_ELT(x_names, i);
 			if (desc == NA_STRING)
 				error("'names(x)' contains NAs");
-			if (fputs(CHAR(desc), stream) == EOF)
-				error("write error");
+			ExternalFilePtr_puts(efp, CHAR(desc));
 		}
-		if (fputs("\n", stream) == EOF)
-			error("write error");
-		X_elt = _get_cachedXStringSet_elt(&X, i);
+		ExternalFilePtr_puts(efp, "\n");
+		X_elt = _get_elt_from_XStringSet_holder(&X, i);
 		for (j1 = 0; j1 < X_elt.length; j1 += width0) {
 			j2 = j1 + width0;
 			if (j2 > X_elt.length)
@@ -434,9 +464,8 @@ SEXP write_XStringSet_to_fasta(SEXP x, SEXP efp_list, SEXP width, SEXP lkup)
 				X_elt.seq, X_elt.length,
 				lkup0, lkup_length);
 			buf[dest_nbytes] = 0;
-			if (fputs(buf, stream) == EOF
-			 || fputs("\n", stream) == EOF)
-				error("write error");
+			ExternalFilePtr_puts(efp, buf);
+			ExternalFilePtr_puts(efp, "\n");
 		}
 	}
 	return R_NilValue;
@@ -457,13 +486,13 @@ static const char *FASTQ_line1_markup = "@", *FASTQ_line3_markup = "+";
 
 typedef struct fastq_loader {
 	void (*load_seqid)(struct fastq_loader *loader,
-			   const cachedCharSeq *seqid);
+			   const Chars_holder *seqid);
 	void (*load_seq)(struct fastq_loader *loader,
-			 const cachedCharSeq *seq);
+			 const Chars_holder *seq);
 	void (*load_qualid)(struct fastq_loader *loader,
-			    const cachedCharSeq *qualid);
+			    const Chars_holder *qualid);
 	void (*load_qual)(struct fastq_loader *loader,
-			  const cachedCharSeq *qual);
+			  const Chars_holder *qual);
 	int nrec;
 	void *ext;  /* loader extension (optional) */
 } FASTQloader;
@@ -477,7 +506,7 @@ typedef struct fastqgeom_loader_ext {
 	int width;
 } FASTQGEOM_loaderExt;
 
-static void FASTQGEOM_load_seq(FASTQloader *loader, const cachedCharSeq *seq)
+static void FASTQGEOM_load_seq(FASTQloader *loader, const Chars_holder *seq)
 {
 	FASTQGEOM_loaderExt *loader_ext;
 
@@ -520,12 +549,12 @@ static FASTQloader new_FASTQGEOM_loader(FASTQGEOM_loaderExt *loader_ext)
 
 typedef struct fastq_loader_ext {
 	CharAEAE ans_names_buf;
-	cachedXVectorList cached_ans;
+	XVectorList_holder ans_holder;
 	const int *lkup;
 	int lkup_length;
 } FASTQ_loaderExt;
 
-static void FASTQ_load_seqid(FASTQloader *loader, const cachedCharSeq *seqid)
+static void FASTQ_load_seqid(FASTQloader *loader, const Chars_holder *seqid)
 {
 	FASTQ_loaderExt *loader_ext;
 	CharAEAE *ans_names_buf;
@@ -537,18 +566,18 @@ static void FASTQ_load_seqid(FASTQloader *loader, const cachedCharSeq *seqid)
 	return;
 }
 
-static void FASTQ_load_seq(FASTQloader *loader, const cachedCharSeq *seq)
+static void FASTQ_load_seq(FASTQloader *loader, const Chars_holder *seq)
 {
 	FASTQ_loaderExt *loader_ext;
-	cachedCharSeq cached_ans_elt;
+	Chars_holder ans_elt_holder;
 
 	loader_ext = loader->ext;
-	cached_ans_elt = get_cachedXRawList_elt(&(loader_ext->cached_ans),
+	ans_elt_holder = get_elt_from_XRawList_holder(&(loader_ext->ans_holder),
 						loader->nrec);
-	/* cached_ans_elt.seq is a const char * so we need to cast it to
+	/* ans_elt_holder.seq is a const char * so we need to cast it to
 	   char * before we can write to it */
-	Ocopy_bytes_to_i1i2_with_lkup(0, cached_ans_elt.length - 1,
-		(char *) cached_ans_elt.seq, cached_ans_elt.length,
+	Ocopy_bytes_to_i1i2_with_lkup(0, ans_elt_holder.length - 1,
+		(char *) ans_elt_holder.seq, ans_elt_holder.length,
 		seq->seq, seq->length,
 		loader_ext->lkup, loader_ext->lkup_length);
 	return;
@@ -560,9 +589,10 @@ static FASTQ_loaderExt new_FASTQ_loaderExt(SEXP ans, SEXP lkup)
 
 	loader_ext.ans_names_buf =
 		new_CharAEAE(_get_XStringSet_length(ans), 0);
-	loader_ext.cached_ans = cache_XVectorList(ans);
+	loader_ext.ans_holder = hold_XVectorList(ans);
 	if (lkup == R_NilValue) {
 		loader_ext.lkup = NULL;
+		loader_ext.lkup_length = 0;
 	} else {
 		loader_ext.lkup = INTEGER(lkup);
 		loader_ext.lkup_length = LENGTH(lkup);
@@ -588,27 +618,44 @@ static FASTQloader new_FASTQ_loader(int load_seqids,
 /*
  * Ignore empty lines.
  */
-static const char *parse_FASTQ_file(FILE *stream, int *recno,
-		int nrec, int skip, FASTQloader *loader)
+static const char *parse_FASTQ_file(SEXP efp, int *recno,
+		int nrec, int skip, int seek_first_rec, FASTQloader *loader)
 {
-	int FASTQ_line1_markup_length, FASTQ_line3_markup_length,
-	    lineno, lineinrecno, load_record;
+	int lineno, EOL_in_buf, EOL_in_prev_buf, ret_code,
+	    FASTQ_line1_markup_length, FASTQ_line3_markup_length,
+	    lineinrecno, load_rec;
 	char buf[IOBUF_SIZE];
-	cachedCharSeq data;
+	Chars_holder data;
 
 	FASTQ_line1_markup_length = strlen(FASTQ_line1_markup);
 	FASTQ_line3_markup_length = strlen(FASTQ_line3_markup);
-	lineno = lineinrecno = 0;
-	while (fgets(buf, IOBUF_SIZE, stream) != NULL) {
-		lineno++;
-		data.length = delete_trailing_LF_or_CRLF(buf, -1);
-		// data.length >= IOBUF_SIZE should never happen
-		if (data.length >= IOBUF_SIZE - 1) {
+	lineinrecno = 0;
+	for (lineno = EOL_in_prev_buf = 1;
+	     (ret_code = ExternalFilePtr_gets(efp, buf, IOBUF_SIZE,
+					      &EOL_in_buf));
+	     lineno += EOL_in_prev_buf = EOL_in_buf)
+	{
+		if (ret_code == -1) {
+			snprintf(errmsg_buf, sizeof(errmsg_buf),
+				 "read error while reading characters "
+				 "from line %d", lineno);
+			return errmsg_buf;
+		}
+		if (seek_first_rec) {
+			if (EOL_in_prev_buf
+			 && has_prefix(buf, FASTQ_line1_markup)) {
+				seek_first_rec = 0;
+			} else {
+				continue;
+			}
+		}
+		if (!EOL_in_buf) {
 			snprintf(errmsg_buf, sizeof(errmsg_buf),
 				 "cannot read line %d, line is too long",
 				 lineno);
 			return errmsg_buf;
 		}
+		data.length = delete_trailing_LF_or_CRLF(buf, -1);
 		if (data.length == 0)
 			continue; // we ignore empty lines
 		buf[data.length] = '\0';
@@ -618,74 +665,77 @@ static const char *parse_FASTQ_file(FILE *stream, int *recno,
 			lineinrecno = 1;
 		switch (lineinrecno) {
 		    case 1:
-			if (strncmp(buf, FASTQ_line1_markup,
-				    FASTQ_line1_markup_length) != 0) {
-			    snprintf(errmsg_buf, sizeof(errmsg_buf),
+			if (!has_prefix(buf, FASTQ_line1_markup)) {
+				snprintf(errmsg_buf, sizeof(errmsg_buf),
 				     "\"%s\" expected at beginning of line %d",
 				     FASTQ_line1_markup, lineno);
 				return errmsg_buf;
 			}
-			load_record = *recno >= skip;
-			if (load_record && nrec >= 0 && *recno >= skip + nrec)
+			load_rec = *recno >= skip;
+			if (load_rec && nrec >= 0 && *recno >= skip + nrec)
 				return NULL;
-			load_record = load_record && loader != NULL;
-			if (load_record && nrec >= 0)
-				load_record = *recno < skip + nrec;
-			if (load_record && loader->load_seqid != NULL) {
+			load_rec = load_rec && loader != NULL;
+			if (load_rec && nrec >= 0)
+				load_rec = *recno < skip + nrec;
+			if (load_rec && loader->load_seqid != NULL) {
 				data.seq += FASTQ_line1_markup_length;
 				data.length -= FASTQ_line1_markup_length;
 				loader->load_seqid(loader, &data);
 			}
 		    break;
 		    case 2:
-			if (load_record && loader->load_seq != NULL)
+			if (load_rec && loader->load_seq != NULL)
 				loader->load_seq(loader, &data);
 		    break;
 		    case 3:
-			if (strncmp(buf, FASTQ_line3_markup,
-				    FASTQ_line3_markup_length) != 0) {
+			if (!has_prefix(buf, FASTQ_line3_markup)) {
 				snprintf(errmsg_buf, sizeof(errmsg_buf),
 					 "\"%s\" expected at beginning of "
 					 "line %d", FASTQ_line3_markup, lineno);
 				return errmsg_buf;
 			}
-			if (load_record && loader->load_qualid != NULL) {
+			if (load_rec && loader->load_qualid != NULL) {
 				data.seq += FASTQ_line3_markup_length;
 				data.length -= FASTQ_line3_markup_length;
 				loader->load_qualid(loader, &data);
 			}
 		    break;
 		    case 4:
-			if (load_record && loader->load_qual != NULL)
+			if (load_rec && loader->load_qual != NULL)
 				loader->load_qual(loader, &data);
-			if (load_record)
+			if (load_rec)
 				loader->nrec++;
 			(*recno)++;
 		    break;
 		}
 	}
+	if (seek_first_rec) {
+		snprintf(errmsg_buf, sizeof(errmsg_buf),
+			 "no FASTQ record found");
+		return errmsg_buf;
+	}
 	return NULL;
 }
 
 /* --- .Call ENTRY POINT --- */
-SEXP fastq_geometry(SEXP efp_list, SEXP nrec, SEXP skip)
+SEXP fastq_geometry(SEXP efp_list, SEXP nrec, SEXP skip, SEXP seek_first_rec)
 {
-	int nrec0, skip0, i, recno;
+	int nrec0, skip0, seek_rec0, i, recno;
 	FASTQGEOM_loaderExt loader_ext;
 	FASTQloader loader;
-	FILE *stream;
 	const char *errmsg;
-	SEXP ans;
+	SEXP efp, ans;
 
 	nrec0 = INTEGER(nrec)[0];
 	skip0 = INTEGER(skip)[0];
+	seek_rec0 = LOGICAL(seek_first_rec)[0];
 	loader_ext = new_FASTQGEOM_loaderExt();
 	loader = new_FASTQGEOM_loader(&loader_ext);
 	recno = 0;
 	for (i = 0; i < LENGTH(efp_list); i++) {
-		stream = R_ExternalPtrAddr(VECTOR_ELT(efp_list, i));
-		errmsg = parse_FASTQ_file(stream, &recno,
-				nrec0, skip0, &loader);
+		efp = VECTOR_ELT(efp_list, i);
+		errmsg = parse_FASTQ_file(efp, &recno,
+					  nrec0, skip0, seek_rec0, &loader);
 		if (errmsg != NULL)
 			error("reading FASTQ file %s: %s",
 			      CHAR(STRING_ELT(GET_NAMES(efp_list), i)),
@@ -700,20 +750,22 @@ SEXP fastq_geometry(SEXP efp_list, SEXP nrec, SEXP skip)
 
 /* --- .Call ENTRY POINT --- */
 SEXP read_fastq_in_XStringSet(SEXP efp_list, SEXP nrec, SEXP skip,
+		SEXP seek_first_rec,
 		SEXP use_names, SEXP elementType, SEXP lkup)
 {
-	int nrec0, skip0, load_seqids, ans_length, i, recno;
-	SEXP ans_geom, ans_width, ans, ans_names;
+	int nrec0, skip0, seek_rec0, load_seqids, ans_length, i, recno;
+	SEXP efp, ans_geom, ans_width, ans, ans_names;
 	const char *element_type;
 	char classname[40];  /* longest string should be "DNAStringSet" */
 	FASTQ_loaderExt loader_ext;
 	FASTQloader loader;
-	FILE *stream;
 
 	nrec0 = INTEGER(nrec)[0];
 	skip0 = INTEGER(skip)[0];
+	seek_rec0 = LOGICAL(seek_first_rec)[0];
 	load_seqids = LOGICAL(use_names)[0];
-	PROTECT(ans_geom = fastq_geometry(efp_list, nrec, skip));
+	PROTECT(ans_geom = fastq_geometry(efp_list, nrec, skip,
+					  seek_first_rec));
 	ans_length = INTEGER(ans_geom)[0];
 	PROTECT(ans_width = NEW_INTEGER(ans_length));
 	if (ans_length != 0) {
@@ -739,9 +791,10 @@ SEXP read_fastq_in_XStringSet(SEXP efp_list, SEXP nrec, SEXP skip,
 	loader = new_FASTQ_loader(load_seqids, &loader_ext);
 	recno = 0;
 	for (i = 0; i < LENGTH(efp_list); i++) {
-		stream = R_ExternalPtrAddr(VECTOR_ELT(efp_list, i));
-		rewind(stream);
-		parse_FASTQ_file(stream, &recno, nrec0, skip0, &loader);
+		efp = VECTOR_ELT(efp_list, i);
+		ExternalFilePtr_rewind(efp);
+		parse_FASTQ_file(efp, &recno,
+				 nrec0, skip0, seek_rec0, &loader);
 	}
 	if (load_seqids) {
 		PROTECT(ans_names =
@@ -789,69 +842,64 @@ static const char *get_FASTQ_rec_id(SEXP x_names, SEXP q_names, int i)
 	return CHAR(seqid);
 }
 
-static int write_FASTQ_id(FILE *stream, const char *markup, const char *id)
+static void write_FASTQ_id(SEXP efp, const char *markup, const char *id)
 {
-	return fputs(markup, stream) == EOF
-	    || fputs(id, stream) == EOF
-	    || fputs("\n", stream) == EOF;
+	ExternalFilePtr_puts(efp, markup);
+	ExternalFilePtr_puts(efp, id);
+	ExternalFilePtr_puts(efp, "\n");
 }
 
-static int write_FASTQ_seq(FILE *stream, const char *buf)
+static void write_FASTQ_seq(SEXP efp, const char *buf)
 {
-	return fputs(buf, stream) == EOF
-	    || fputs("\n", stream) == EOF;
+	ExternalFilePtr_puts(efp, buf);
+	ExternalFilePtr_puts(efp, "\n");
 }
 
-static int write_FASTQ_qual(FILE *stream, int seqlen,
-		const cachedXStringSet *Q, int i)
+static void write_FASTQ_qual(SEXP efp, int seqlen,
+		const XStringSet_holder *Q, int i)
 {
-	cachedCharSeq Q_elt;
+	Chars_holder Q_elt;
 	int j;
 
-	Q_elt = _get_cachedXStringSet_elt(Q, i);
+	Q_elt = _get_elt_from_XStringSet_holder(Q, i);
 	if (Q_elt.length != seqlen)
 		error("'x' and 'quality' must have the same width");
-	for (j = 0; j < seqlen; j++) {
-		if (fputc((int) *(Q_elt.seq++), stream) == EOF)
-			return 1;
-	}
-	return fputs("\n", stream) == EOF;
+	for (j = 0; j < seqlen; j++)
+		ExternalFilePtr_putc(efp, (int) *(Q_elt.seq++));
+	ExternalFilePtr_puts(efp, "\n");
 }
 
-static int write_FASTQ_fakequal(FILE *stream, int seqlen)
+static void write_FASTQ_fakequal(SEXP efp, int seqlen)
 {
 	int j;
 
-	for (j = 0; j < seqlen; j++) {
-		if (fputc((int) ';', stream) == EOF)
-			return 1;
-	}
-	return fputs("\n", stream) == EOF;
+	for (j = 0; j < seqlen; j++)
+		ExternalFilePtr_putc(efp, (int) ';');
+	ExternalFilePtr_puts(efp, "\n");
 }
 
 /* --- .Call ENTRY POINT --- */
 SEXP write_XStringSet_to_fastq(SEXP x, SEXP efp_list, SEXP qualities, SEXP lkup)
 {
-	cachedXStringSet X, Q;
+	XStringSet_holder X, Q;
 	int x_length, lkup_length, i;
-	FILE *stream;
 	const int *lkup0;
-	SEXP x_names, q_names;
+	SEXP efp, x_names, q_names;
 	const char *id;
-	cachedCharSeq X_elt;
+	Chars_holder X_elt;
 	char buf[IOBUF_SIZE];
 
-	X = _cache_XStringSet(x);
-	x_length = _get_cachedXStringSet_length(&X);
+	X = _hold_XStringSet(x);
+	x_length = _get_length_from_XStringSet_holder(&X);
 	if (qualities != R_NilValue) {
-		Q = _cache_XStringSet(qualities);
-		if (_get_cachedXStringSet_length(&Q) != x_length)
+		Q = _hold_XStringSet(qualities);
+		if (_get_length_from_XStringSet_holder(&Q) != x_length)
 			error("'x' and 'qualities' must have the same length");
 		q_names = get_XVectorList_names(qualities);
 	} else {
 		q_names = R_NilValue;
 	}
-	stream = R_ExternalPtrAddr(VECTOR_ELT(efp_list, 0));
+	efp = VECTOR_ELT(efp_list, 0);
 	if (lkup == R_NilValue) {
 		lkup0 = NULL;
 		lkup_length = 0;
@@ -862,22 +910,19 @@ SEXP write_XStringSet_to_fastq(SEXP x, SEXP efp_list, SEXP qualities, SEXP lkup)
 	x_names = get_XVectorList_names(x);
 	for (i = 0; i < x_length; i++) {
 		id = get_FASTQ_rec_id(x_names, q_names, i);
-		X_elt = _get_cachedXStringSet_elt(&X, i);
+		X_elt = _get_elt_from_XStringSet_holder(&X, i);
 		Ocopy_bytes_from_i1i2_with_lkup(0, X_elt.length - 1,
 			buf, X_elt.length,
 			X_elt.seq, X_elt.length,
 			lkup0, lkup_length);
 		buf[X_elt.length] = 0;
-		if (write_FASTQ_id(stream, FASTQ_line1_markup, id)
-		 || write_FASTQ_seq(stream, buf)
-		 || write_FASTQ_id(stream, FASTQ_line3_markup, id))
-			error("write error");
+		write_FASTQ_id(efp, FASTQ_line1_markup, id);
+		write_FASTQ_seq(efp, buf);
+		write_FASTQ_id(efp, FASTQ_line3_markup, id);
 		if (qualities != R_NilValue) {
-			if (write_FASTQ_qual(stream, X_elt.length, &Q, i))
-				error("write error");
+			write_FASTQ_qual(efp, X_elt.length, &Q, i);
 		} else {
-			if (write_FASTQ_fakequal(stream, X_elt.length))
-				error("write error");
+			write_FASTQ_fakequal(efp, X_elt.length);
 		}
 	}
 	return R_NilValue;
